@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/in-memory.lisp,v 1.26 2007/12/29 21:17:05 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/in-memory.lisp,v 1.31 2008/05/19 07:57:07 edi Exp $
 
-;;; Copyright (c) 2005-2007, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2005-2008, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -104,178 +104,196 @@ associated vector of octets."))
   (:documentation "A binary output stream that writes its data to an
 associated vector."))
 
-(define-condition in-memory-stream-error (stream-error)
-  ()
-  (:documentation "Superclass for all errors related to
-IN-MEMORY streams."))
-
-(define-condition in-memory-stream-closed-error (in-memory-stream-error)
-  ()
-  (:report (lambda (condition stream)
-             (format stream "~S is closed."
-                     (stream-error-stream condition))))
-  (:documentation "An error that is signalled when someone is trying
-to read from or write to a closed IN-MEMORY stream."))
-
 #+:cmu
 (defmethod open-stream-p ((stream in-memory-stream))
   "Returns a true value if STREAM is open.  See ANSI standard."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (in-memory-stream-open-p stream))
 
 #+:cmu
 (defmethod close ((stream in-memory-stream) &key abort)
   "Closes the stream STREAM.  See ANSI standard."
-  (declare (ignore abort)
-           (optimize speed))
+  (declare #.*standard-optimize-settings*)
+  (declare (ignore abort))
   (prog1
       (in-memory-stream-open-p stream)
     (setf (in-memory-stream-open-p stream) nil)))
 
 (defmethod check-if-open ((stream in-memory-stream))
   "Checks if STREAM is open and signals an error otherwise."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (unless (open-stream-p stream)
     (error 'in-memory-stream-closed-error
            :stream stream)))
 
 (defmethod stream-element-type ((stream in-memory-stream))
   "The element type is always OCTET by definition."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   'octet)
 
 (defmethod transform-octet ((stream in-memory-stream) octet)
   "Applies the transformer of STREAM to octet and returns the result."
+  (declare #.*standard-optimize-settings*)
   (funcall (or (in-memory-stream-transformer stream)
                #'identity) octet))
 
 (defmethod stream-read-byte ((stream list-input-stream))
   "Reads one byte by simply popping it off of the top of the list."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (check-if-open stream)
-  (transform-octet stream (or (pop (list-stream-list stream))
-                              (return-from stream-read-byte :eof))))
+  (with-accessors ((list list-stream-list))
+      stream
+    (transform-octet stream (or (pop list) (return-from stream-read-byte :eof)))))
 
 (defmethod stream-listen ((stream list-input-stream))
   "Checks whether list is not empty."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (check-if-open stream)
-  (list-stream-list stream))
+  (with-accessors ((list list-stream-list))
+      stream
+    list))
 
 (defmethod stream-read-sequence ((stream list-input-stream) sequence start end &key)
   "Repeatedly pops elements from the list until it's empty."
-  (declare (optimize speed) (type (integer 0 *) start end))
-  (loop for index from start below end
-        while (list-stream-list stream)
-        do (setf (elt sequence index)
-                 (pop (list-stream-list stream)))
-        finally (return index)))
+  (declare #.*standard-optimize-settings*)
+  (declare (fixnum start end))
+  (with-accessors ((list list-stream-list))
+      stream
+    (loop for index of-type fixnum from start below end
+          while list
+          do (setf (elt sequence index) (pop list))
+          finally (return index))))
 
 (defmethod stream-read-byte ((stream vector-input-stream))
   "Reads one byte and increments INDEX pointer unless we're beyond
 END pointer."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (check-if-open stream)
-  (let ((index (vector-stream-index stream)))
-    (cond ((< index (vector-stream-end stream))
-           (incf (vector-stream-index stream))
-           (transform-octet stream (aref (vector-stream-vector stream) index)))
-          (t :eof))))
+  (with-accessors ((index vector-stream-index)
+                   (end vector-stream-end)
+                   (vector vector-stream-vector))
+      stream
+    (let ((current-index index))
+      (declare (fixnum current-index))
+      (cond ((< current-index (the fixnum end))
+             (incf (the fixnum index))
+             (transform-octet stream (aref vector current-index)))
+            (t :eof)))))
 
 (defmethod stream-listen ((stream vector-input-stream))
   "Checking whether INDEX is beyond END."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (check-if-open stream)
-  (< (vector-stream-index stream) (vector-stream-end stream)))
+  (with-accessors ((index vector-stream-index)
+                   (end vector-stream-end))
+      stream
+    (< (the fixnum index) (the fixnum end))))
   
 (defmethod stream-read-sequence ((stream vector-input-stream) sequence start end &key)
   "Traverses both sequences in parallel until the end of one of them
 is reached."
-  (declare (optimize speed) (type (integer 0 *) start end))
-  (loop with vector-end of-type (integer 0 #.array-dimension-limit) = (vector-stream-end stream)
+  (declare #.*standard-optimize-settings*)
+  (declare (fixnum start end))
+  (loop with vector-end of-type fixnum = (vector-stream-end stream)
         with vector = (vector-stream-vector stream)
-        for index from start below end
-        for vector-index of-type (integer 0 #.array-dimension-limit) = (vector-stream-index stream)
+        for index of-type fixnum from start below end
+        for vector-index of-type fixnum = (vector-stream-index stream)
         while (< vector-index vector-end)
         do (setf (elt sequence index)
                  (aref vector vector-index))
-           (incf (vector-stream-index stream))
+           (incf (the fixnum (vector-stream-index stream)))
         finally (return index)))
 
 (defmethod stream-write-byte ((stream vector-output-stream) byte)
   "Writes a byte \(octet) by extending the underlying vector."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (check-if-open stream)
-  (vector-push-extend (transform-octet stream byte)
-                      (vector-stream-vector stream)))
+  (with-accessors ((vector vector-stream-vector))
+      stream
+    (vector-push-extend (transform-octet stream byte) vector)))
 
 (defmethod stream-write-sequence ((stream vector-output-stream) sequence start end &key)
   "Just calls VECTOR-PUSH-EXTEND repeatedly."
-  (declare (optimize speed) (type (integer 0 *) start end))
-  (loop with vector = (vector-stream-vector stream)
-        for index from start below end
-        do (vector-push-extend (elt sequence index) vector))
-  sequence)
+  (declare #.*standard-optimize-settings*)
+  (declare (fixnum start end))
+  (with-accessors ((vector vector-stream-vector))
+      stream
+    (loop for index of-type fixnum from start below end
+          do (vector-push-extend (elt sequence index) vector))
+    sequence))
 
 (defmethod stream-file-position ((stream vector-input-stream))
   "Simply returns the index into the underlying vector."
-  (declare (optimize speed))
-  (vector-stream-index stream))
+  (declare #.*standard-optimize-settings*)
+  (with-accessors ((index vector-stream-index))
+      stream
+    index))
 
 (defmethod (setf stream-file-position) (position-spec (stream vector-input-stream))
   "Sets the index into the underlying vector if POSITION-SPEC is acceptable."
-  (declare (optimize speed))
-  (setf (vector-stream-index stream)
-        (case position-spec
-          (:start 0)
-          (:end (vector-stream-end stream))
-          (otherwise
-           (unless (integerp position-spec)
-             (error 'flexi-stream-position-spec-error
-                    :format-control "Unknown file position designator: ~S."
-                    :format-arguments (list position-spec)
-                    :position-spec position-spec))
-           (unless (<= 0 position-spec (vector-stream-end stream))
-             (error 'flexi-stream-position-spec-error
-                    :format-control "File position designator ~S is out of bounds."
-                    :format-arguments (list position-spec)
-                    :position-spec position-spec))
-           position-spec)))
-  position-spec)
+  (declare #.*standard-optimize-settings*)
+  (with-accessors ((index vector-stream-index)
+                   (end vector-stream-end))
+      stream
+    (setq index
+          (case position-spec
+            (:start 0)
+            (:end end)
+            (otherwise
+             (unless (integerp position-spec)
+               (error 'in-memory-stream-position-spec-error
+                      :format-control "Unknown file position designator: ~S."
+                      :format-arguments (list position-spec)
+                      :stream stream
+                      :position-spec position-spec))
+             (unless (<= 0 position-spec end)
+               (error 'in-memory-stream-position-spec-error
+                      :format-control "File position designator ~S is out of bounds."
+                      :format-arguments (list position-spec)
+                      :stream stream
+                      :position-spec position-spec))
+             position-spec)))
+    position-spec))
 
 (defmethod stream-file-position ((stream vector-output-stream))
   "Simply returns the fill pointer of the underlying vector."
-  (declare (optimize speed))
-  (fill-pointer (vector-stream-vector stream)))
+  (declare #.*standard-optimize-settings*)
+  (with-accessors ((vector vector-stream-vector))
+      stream
+    (fill-pointer vector)))
 
 (defmethod (setf stream-file-position) (position-spec (stream vector-output-stream))
   "Sets the fill pointer underlying vector if POSITION-SPEC is
 acceptable.  Adjusts the vector if necessary."
-  (declare (optimize speed))
-  (let* ((vector (vector-stream-vector stream))
-         (total-size (array-total-size vector))
-         (new-fill-pointer
-          (case position-spec
-            (:start 0)
-            (:end
-             (warn "File position designator :END doesn't really make sense for an output stream.")
-             total-size)
-            (otherwise
-             (unless (integerp position-spec)
-               (error 'flexi-stream-position-spec-error
-                      :format-control "Unknown file position designator: ~S."
-                      :format-arguments (list position-spec)
-                      :position-spec position-spec))
-             (unless (<= 0 position-spec array-total-size-limit)
-               (error 'flexi-stream-position-spec-error
-                      :format-control "File position designator ~S is out of bounds."
-                      :format-arguments (list position-spec)
-                      :position-spec position-spec))
-             position-spec))))
-    (when (> new-fill-pointer total-size)
-      (adjust-array vector new-fill-pointer))
-    (setf (fill-pointer vector) new-fill-pointer)
-    position-spec))
+  (declare #.*standard-optimize-settings*)
+  (with-accessors ((vector vector-stream-vector))
+      stream
+    (let* ((total-size (array-total-size vector))
+           (new-fill-pointer
+            (case position-spec
+              (:start 0)
+              (:end
+               (warn "File position designator :END doesn't really make sense for an output stream.")
+               total-size)
+              (otherwise
+               (unless (integerp position-spec)
+                 (error 'in-memory-stream-position-spec-error
+                        :format-control "Unknown file position designator: ~S."
+                        :format-arguments (list position-spec)
+                        :stream stream
+                        :position-spec position-spec))
+               (unless (<= 0 position-spec array-total-size-limit)
+                 (error 'in-memory-stream-position-spec-error
+                        :format-control "File position designator ~S is out of bounds."
+                        :format-arguments (list position-spec)
+                        :stream stream
+                        :position-spec position-spec))
+               position-spec))))
+      (declare (fixnum total-size new-fill-pointer))
+      (when (> new-fill-pointer total-size)
+        (adjust-array vector new-fill-pointer))
+      (setf (fill-pointer vector) new-fill-pointer)
+      position-spec)))
 
 (defmethod make-in-memory-input-stream ((vector vector) &key (start 0)
                                                              (end (length vector))
@@ -284,7 +302,7 @@ acceptable.  Adjusts the vector if necessary."
 octets in the subsequence of VECTOR bounded by START and END.
 Each octet returned will be transformed in turn by the optional
 TRANSFORMER function."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (make-instance 'vector-input-stream
                  :vector vector
                  :index start
@@ -298,7 +316,7 @@ TRANSFORMER function."
 octets in the subsequence of LIST bounded by START and END.  Each
 octet returned will be transformed in turn by the optional
 TRANSFORMER function."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (make-instance 'list-input-stream
                  :list (subseq list start end)
                  :transformer transformer))
@@ -306,7 +324,7 @@ TRANSFORMER function."
 (defun make-output-vector (&key (element-type 'octet))
   "Creates and returns an array which can be used as the underlying
 vector for a VECTOR-OUTPUT-STREAM."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (make-array 0 :adjustable t
                 :fill-pointer 0
                 :element-type element-type))
@@ -317,7 +335,7 @@ ELEMENT-TYPE \(a subtype of OCTET) and makes available a sequence
 that contains the octes that were actually output.  The octets
 stored will each be transformed by the optional TRANSFORMER
 function."
-  (declare (optimize speed))
+  (declare #.*standard-optimize-settings*)
   (make-instance 'vector-output-stream
                  :vector (make-output-vector :element-type element-type)
                  :transformer transformer))
@@ -329,19 +347,23 @@ octets on STREAM, so the vector contains only those octets which have
 been output since the last call to GET-OUTPUT-STREAM-SEQUENCE or since
 the creation of the stream, whichever occurred most recently.  If
 AS-LIST is true the return value is coerced to a list."
-  (declare (optimize speed))
-  (prog1
-      (if as-list
-        (coerce (vector-stream-vector stream) 'list)
-        (vector-stream-vector stream))
-    (setf (vector-stream-vector stream)
-          (make-output-vector))))
+  (declare #.*standard-optimize-settings*)
+  (with-accessors ((vector vector-stream-vector))
+      stream
+    (prog1
+        (if as-list
+          (coerce vector 'list)
+          vector)
+      (setq vector
+            (make-output-vector)))))
 
 (defmethod output-stream-sequence-length ((stream in-memory-output-stream))
   "Returns the current length of the underlying vector of the
 IN-MEMORY output stream STREAM."
   (declare (optimize speed))
-  (length (the (simple-array * (*)) (vector-stream-vector stream))))
+  (with-accessors ((vector vector-stream-vector))
+      stream
+    (length (the (simple-array * (*)) vector))))
 
 (defmacro with-input-from-sequence ((var sequence &key start end transformer) 
                                     &body body)
@@ -382,14 +404,3 @@ stream within BODY."
            ,@body
            (get-output-stream-sequence ,var :as-list ,as-list))
        (when ,var (close ,var)))))
-
-(declaim (inline translate-char))
-(defun translate-char (char-code external-format)
-  "Returns a list of octets which correspond to the
-representation of the character with character code CHAR-CODE
-when sent to a flexi stream with external format EXTERNAL-FORMAT.
-Used internally by UNREAD-CHAR%.  See also STRING-TO-OCTETS."
-  (declare (optimize speed))
-  (with-output-to-sequence (list :as-list t)
-    (let ((stream (make-flexi-stream list :external-format external-format)))
-      (write-char (code-char char-code) stream))))
\ No newline at end of file
