@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/util.lisp,v 1.13 2007/01/01 23:46:49 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/util.lisp,v 1.24 2008/05/25 21:26:12 edi Exp $
 
-;;; Copyright (c) 2005-2007, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2005-2008, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -31,7 +31,14 @@
 
 #+:lispworks
 (eval-when (:compile-toplevel :load-toplevel :execute)
-  (import 'lw:with-unique-names))
+  (import '(lw:with-unique-names lw:when-let)))
+
+#-:lispworks
+(defmacro when-let ((var form) &body body)
+  "Evaluates FORM and binds VAR to the result, then executes BODY
+if VAR has a true value."
+  `(let ((,var ,form))
+     (when ,var ,@body)))
 
 #-:lispworks
 (defmacro with-unique-names ((&rest bindings) &body body)
@@ -108,7 +115,9 @@ signals an error otherwise."
     (unless (find real-name +name-map+
                   :test #'eq
                   :key #'cdr)
-      (error "~S is not known to be a name for an external format." name))
+      (error 'external-format-error
+             :format-control "~S is not known to be a name for an external format."
+             :format-arguments (list name)))
     real-name))
 
 (defun ascii-name-p (name)
@@ -161,6 +170,38 @@ code pages."
 (defmacro with-accessors (slot-entries instance &body body)
   "For LispWorks, we prefer SLOT-VALUE over accessors for better
 performance."
+  ;; note that we assume that the variables have the same names as the
+  ;; slots
   `(with-slots ,(mapcar #'car slot-entries)
        ,instance
-     ,@body))
\ No newline at end of file
+     ,@body))
+
+(defun make-octet-buffer (&optional (size +buffer-size+))
+  "Creates and returns a fresh buffer \(a specialized array) of size
++BUFFER-SIZE+ to hold octets."
+  (declare #.*standard-optimize-settings*)
+  (make-array size :element-type 'octet))
+
+(defun type-equal (type1 type2)
+  "Whether TYPE1 and TYPE2 denote the same type."
+  (declare #.*standard-optimize-settings*)
+  (and (subtypep type1 type2)
+       (subtypep type2 type1)))
+
+(defun maybe-rewind (stream octets)
+  "Tries to `rewind' the \(binary) stream STREAM by OCTETS octets.
+Returns a true value if it succeeds."
+  (when-let (position (file-position stream))
+    (file-position stream (- position octets))))
+
+(defmacro logand* (x y)
+  "Solely for optimization purposes.  Some Lisps need it, some don't."
+  `(the fixnum (logand ,x ,y)))
+
+(defmacro logior* (x y)
+  "Solely for optimization purposes.  Some Lisps need it, some don't."
+  `(the fixnum (logior ,x ,y)))
+
+(defmacro ash* (integer count)
+  "Solely for optimization purposes.  Some Lisps need it, some don't."
+  `(the fixnum (ash ,integer ,count)))
